use strict;

package XML::Generator::BostonGlobe;
use base qw (XML::SAX::Base);

$XML::Generator::BostonGlobe::VERSION = '0.1';
# $Id: BostonGlobe.pm,v 1.3 2004/07/27 17:27:46 asc Exp $

=head1 NAME

XML::Generator::BostonGlobe - generate SAX2 events for bostonglobe.com documents

=head1 SYNOPSIS

 use XML::Generator::BostonGlobe;
 
 use XML::SAX::Writer;
 use XML::Filter::Data::Indenter;

 use IO::AtomicSimple;
 use Date::Simple;

 my $date    = Date::Simple->new();

 my $io      = IO::AtomicFile->open("/some/file","w");
 my $writer  = XML::SAX::Writer->new(Output=>$io);
 my $handler = XML::Filter::DataIndenter->new(Handler=>$writer);

 my $globe = XML::Generator::BostonGlobe->new(Handler=>$handler);
 my $ok    = $globe->fetch($date);

    if ($ok) {
	$io->close();
    }

=head1 DESCRIPTION

Generate SAX2 events for bostonglobe.com documents.

=head1 TERMS OF USAGE

It's not at all clear what the exact Terms of Usage are for the Boston
Globe. However since every article is copyrighted the New York Times, 
this software is meant for personal use only, in accordance with the 
New York Times terms of usage :

 "You may download or copy the Content and other downloadable 
  items displayed on the Service for personal use only, provided
  that you maintain all copyright and other notices contained therein.
  Copying or storing of any Content for other than personal use is 
  expressly prohibited without prior written permission from The New 
  York Times Rights and Permissions Department, or the copyright holder
  identified in the copyright notice contained in the Content."

  http://www.nytimes.com/ref/membercenter/help/agree.html#sect2

Play nicely.

=head1 DOCUMENT STRUCTURE

This package generates DocBook SAX2 events with either a I<book>
or an I<article> root element.

Book widgets contain the following elements :

 + book
   + bookinfo
     - title
     - pubdate
   + part (+)
     + partinfo
       - title
     + xref (+)
       @linkend
     + article (+)

Article widgets contain the following elements :

 + article
   @id
   + articleinfo
     - title
     + keywordset (not yet, anyway)
       - keyword
     + indexterm (not yet, anyway)
       @ type
       - primary
     + authorgroup
       - corpauthor
       OR
       + author
        - surname
        - firstname
        - othername (or not)
     + abstract
       - para
     - pubdate
     + copyright
       - year
       - holder
   - para (+)

=cut

use WWW::Mechanize;
use Date::Simple;
use XML::Simple;

use constant URL_GLOBE_AZ =>
    "http://www.boston.com/news/globe/a-z/";

use constant COPY_HOLDER => "The New York Times Company";

use constant DATE_FORMAT_FS      => "%Y/%m/%d";
use constant DATE_FORMAT_DISPLAY => "%A %B %d, %Y";    

=head1 PACKAGE METHODS

=cut

=head2 __PACKAGE__->new(%args)

=cut

sub new {
    my $pkg  = shift;
    my $self = $pkg->SUPER::new(@_);

    $self->{'__www'} = WWW::Mechanize->new();
    return bless $self, $pkg;
}

=head1 OBJECT METHODS

=cut

=head2 $obj->fetch(($date || Date::Simple))

=cut

sub fetch {
    my $self = shift;
    my $date = shift;

    unless (ref($date) eq "Date::Simple") {
	$date = Date::Simple->new($date);
    }

    $self->{'__date'} = $date;

    #

    $self->_getlinks($date);

    $self->start_document();
    $self->xml_decl({Version=>"1.0",Encoding=>"ISO-8859-1"});
    
    $self->start_element({Name=>"book"});
    $self->start_element({Name=>"bookinfo"});
    $self->start_element({Name=>"title"});
    $self->characters({Data=>sprintf("The Boston Globe for %s",
				     $self->{'__date'}->format(DATE_FORMAT_DISPLAY))});
    $self->end_element({Name=>"title"});
    $self->start_element({Name=>"pubdate"});
    $self->characters({Data=>$self->{'__date'}->format(DATE_FORMAT_FS)});
    $self->end_element({Name=>"pubdate"});
    $self->end_element({Name=>"bookinfo"});
    
    foreach my $sect ($self->_sections()) { 
	$self->_sect($sect);
    }

    $self->end_element({Name=>"book"});
    $self->end_document();

    return 1;
}

sub _sections {
    my $self = shift;
    return sort {$a cmp $b } keys %{$self->{"__data"}};
}

sub _sect {
    my $self = shift;
    my $sect = shift;
    
    my $title = $sect;
    
    $self->start_element({Name=>"part"});
    $self->start_element({Name=>"partinfo"});
    $self->start_element({Name=>"title"});
    $self->characters({Data=>$title});
    $self->end_element({Name=>"title"});
    $self->end_element({Name=>"partinfo"});
    
    foreach my $a ($self->_articles($sect)) {
	
	if ($self->{'__list'}->{$a}) {
	    $self->start_element({Name=>"xref",Attributes=>{
		"{}linkend" => {
		    Name         => "linkend",
		    LocalName    => "linkend",
		    NamespaceURI => "",
		    Prefix       => "",
		    Value        => $a,
		}
	    }});
	    $self->end_element({Name=>"xref"});
	    
	    next;
	}
	
	$self->{'__list'}->{$a} ++;
	
	#
	
	$self->_article($a);
    }
    
    $self->end_element({Name=>"part"});
    return 1;
}

sub _articles {
    my $self = shift;
    my $sect = shift;

    return keys %{$self->{"__data"}->{ $sect }};
}

sub _article {
    my $self = shift;
    my $url  = shift;

    my $url_print = sprintf("%s?mode=PF",$url);
    $self->{'__www'}->get($url_print);

    my $parser = HTML::Parser->new(start_h   => [\&_start,
						 "self,tagname,attr"],
				   end_h     => [\&_end,
						 "self,tagname"],
				   text_h    => [\&_text,
						 "self,text"],
				   comment_h => [\&_comment,
						 "self,text"]);
    
    $parser->parse($self->{'__www'}->content());

    #

    $self->start_element({Name=>"article",Attributes=>{"{}id" => {
	Name         => "id",
	LocalName    => "id",
	Prefix       => "",
	NamespaceURI => "",
	Value        => $url,},
						   }});
    
    $self->start_element({Name=>"articleinfo"});
    $self->start_element({Name=>"title"});
    $self->characters({Data=>$parser->{'__title'}});
    $self->end_element({Name=>"title"});
    $self->start_element({Name=>"abstract"});
    $self->start_element({Name=>"para"});
    $self->characters({Data=>$parser->{'__abstract'}});
    $self->end_element({Name=>"para"});
    $self->end_element({Name=>"abstract"});
    $self->start_element({Name=>"author"});
    $self->start_element({Name=>"personname"});
    $self->characters({Data=>$parser->{'__byline'}});
    $self->end_element({Name=>"personname"});
    $self->end_element({Name=>"author"});
    $self->start_element({Name=>"pubdate"});
    $self->characters({Data=>$parser->{'__date'}});
    $self->end_element({Name=>"pubdate"});
    $self->start_element({Name=>"copyright"});
    $self->start_element({Name=>"year"});
    $self->characters({Data=>$self->{'__date'}->format("%Y")});
    $self->end_element({Name=>"year"});
    $self->start_element({Name=>"holder"});
    $self->characters({Data=>COPY_HOLDER});
    $self->end_element({Name=>"holder"});
    $self->end_element({Name=>"copyright"});
    $self->end_element({Name=>"articleinfo"});
    
    map { 
	$self->start_element({Name=>"para"});
	$self->characters({Data=>$_});
	$self->end_element({Name=>"para"});
    } grep { 
	! /^$/; 
    } @{$parser->{'__body'}};
    
    #
    
    $self->end_element({Name=>"article"});
    return 1;
}

sub _getlinks {
    my $self = shift;
    my $date = shift;

    $self->{'__www'}->get(URL_GLOBE_AZ);

    my %data   = ();
    my $re_date = $date->format("%Y/%m/%d");

    map {
	
	my $url = $_->url_abs();
	$url=~ m!http://www.boston.com/([\w/]+)/articles/$re_date/(?:[^/]+)$!;

	my $sect = $1;
	$sect ||= "unknown";

	$data{ $sect }->{ $url } ++;
    } $self->{'__www'}->find_all_links(url_regex=>qr!articles/$re_date!);

    #

    $self->{"__data"} = \%data;
    return 1;
}

sub _start {
    my $parser = shift;
    my $tag    = shift;
    my $attrs  = shift;

    # print STDERR "[$tag] $attrs->{class}\n";

    if (($tag eq "div") && ($attrs->{class} eq "story")) {
	$parser->{'__start'} = 1;
	return 1;
    }

    if (! $parser->{'__start'}) {
	return 0;
    }

    if (($tag eq "span") && ($attrs->{class} eq "tagline")) {
	$parser->{'__start'} = 0;
	return 1;
    }

    if (($tag eq "p") && ($attrs->{class} ne "byline")) {
	$parser->{"__ok"} = 1;
    }

    return 1;
}

sub _end {
    my $parser = shift;
    my $tag    = shift;

    if (! $parser->{'__start'}) {
	return 0;
    }

    if (! $parser->{'__ok'}) {
	return 0;
    }

    if ($tag eq "p") {
	
	if ($parser->{"__text"}) {
	    $parser->{"__body"} ||= [];
	    push @{$parser->{"__body"}},join("",@{$parser->{"__text"}});	   
	}

	delete $parser->{"__text"};
	$parser->{"__ok"} = 1;
    }
}

sub _text {
    my $parser = shift;
    my $text   = shift;
    
    if (! $parser->{'__ok'}) {
	return 0;
    }

    #

    chomp($text);
    
    $text =~ s/^\s+//;
    $text =~ s/\s+$//;
    
    if (! $text) {
	return 0;
    }

    $parser->{"__text"} ||= [];
    push @{$parser->{"__text"}}, $text;
}

sub _comment {
    my $parser = shift;
    my $tokens = shift;
    
    if (ref($tokens) ne "ARRAY") {
	$tokens = [ $tokens ];
    }

    for (@$tokens) {
	&_token($parser,$_);
    }
}

sub _token {
    my $parser = shift;
    my $tok    = shift;

    $tok =~ s/^<\!--//;
    $tok =~ s/-->$//;

    $tok =~ s/^(\s+)//;
    $tok =~ s/(\s+)$//;

    if ($tok !~ /^<headline/) {
	return 0;
    }

    my $xml = XMLin("<meta>$tok</meta>") || die $!;

    $parser->{ "__title" }    = $xml->{ "headline" };
    $parser->{ "__byline" }   = $xml->{ "byline" };
    $parser->{ "__abstract" } = $xml->{ "teasetext" };
    $parser->{ "__date" }     = $xml->{ "date" };
    
    if (ref($parser->{"__byline" })) {
	$parser->{ "__byline" }	= $xml->{ "source" };
    }

    return 1;
}

=head1 VERSION

0.1

=head1 DATE

$Date: 2004/07/27 17:27:46 $

=head1 AUTHOR

Aaron Straup Cope E<lt>ascope@cpan.orgE<gt>

=head1 LICENSE

Copyright (c) 2004 Aaron Straup Cope. All Rights Reserved.

This is free software, you may use it and distribute it under
the same terms as Perl itself.

=cut

return 1;

__END__
